package com.unis.project.server.controller;

import com.github.pagehelper.PageInfo;
import com.unis.common.utils.ResponseWrapper;
import com.unis.project.common.vo.ProjectQueryVO;
import com.unis.project.common.vo.ProjectVO;
import com.unis.project.server.service.ProjectService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author WangZL
 * @version 1.0
 * @Title ProjectController
 * @ProjectName 项目信息 ProjectController
 * @Description  TODO
 * @date 2019/07/22 17点26分
 */
@RestController
@Api(value = "项目信息 ", tags = "项目信息")
@Slf4j
public class ProjectController {

	@Autowired
	private ProjectService projectService;

	/**
	 * @author WangZL
	 * @Description 项目信息-查询列表
	 * @param 项目方案标题，项目年度
	 * @return 项目信息列表
	 */
	@ApiOperation(value = "项目信息-查询列表", notes = "参数：项目方案标题，项目年度")
	@PostMapping(value = "/projectList")
	public ResponseWrapper getProjectList(@RequestBody ProjectQueryVO projectQueryVO){
		try {
			List<ProjectVO> projectList = projectService.getProjectList(projectQueryVO);
			return ResponseWrapper.success(HttpStatus.OK.value(), HttpStatus.OK.getReasonPhrase(), projectList);
		} catch (Exception e) {
			e.printStackTrace();
			log.error(e.getMessage());
		}
		return ResponseWrapper.error(HttpStatus.INTERNAL_SERVER_ERROR.value(), HttpStatus.INTERNAL_SERVER_ERROR.getReasonPhrase(),"服务错误，请联系管理员");
	}

	/**
	 * @author WangZL
	 * @Description 项目信息-分页查询列表
	 * @param 项目方案标题，项目年度
	 * @return
	 */
	@ApiOperation(value = "项目信息-分页查询列表", notes = "参数：项目方案标题，项目年度")
	@PostMapping(value = "/projectPage/{pageNum}/{pageSize}")
	public ResponseWrapper getProjectPage(@RequestBody ProjectQueryVO projectQueryVO ,@PathVariable("pageNum")Integer pageNum,@PathVariable("pageSize") Integer pageSize) {
		try{
			PageInfo<ProjectVO> pageInfos = new PageInfo<ProjectVO>(projectService.getProjectPage(projectQueryVO,pageNum,pageSize));
			return ResponseWrapper.success(HttpStatus.OK.value(), HttpStatus.OK.getReasonPhrase(), pageInfos);
		} catch (Exception e) {
			e.printStackTrace();
			log.error(e.getMessage());
		}
		return ResponseWrapper.error(HttpStatus.INTERNAL_SERVER_ERROR.value(), HttpStatus.INTERNAL_SERVER_ERROR.getReasonPhrase(),"服务错误，请联系管理员");
	}

	/**
	 * @author WangZL
	 * @Description 项目信息-新增
	 * @param 项目信息实体 models
	 * @return 新增状态
	 */
	@PostMapping(value = "/project")
	@ApiOperation(value = "项目信息-新增", notes = "参数：项目信息对象 model")
	public ResponseWrapper insert(@RequestBody ProjectVO projectVO) {
		try{
			ProjectVO projectVOs = projectService.insert(projectVO);
			if(projectVOs == null) {
				return ResponseWrapper.success(HttpStatus.BAD_REQUEST.value(), "新增项目信息失败，请重新输入", null);
			}
			return ResponseWrapper.success(HttpStatus.OK.value(), HttpStatus.OK.getReasonPhrase(), projectVOs);
		} catch (Exception e) {
			e.printStackTrace();
			log.error(e.getMessage());
		}
		return ResponseWrapper.error(HttpStatus.INTERNAL_SERVER_ERROR.value(), HttpStatus.INTERNAL_SERVER_ERROR.getReasonPhrase(),"服务错误，请联系管理员");
	}

	/**
	 * @author WangZL
	 * @Description 项目信息-更新
	 * @param 项目信息对象 model
	 * @return 新增状态
	 */
	@PutMapping(value = "/project")
	@ApiOperation(value = "项目信息-更新", notes = "参数：项目信息对象 model")
	public ResponseWrapper update(@RequestBody ProjectVO projectVO) {
		try{
			int x = projectService.update(projectVO);
			if(x>0) {
				return ResponseWrapper.success(HttpStatus.OK.value(), HttpStatus.OK.getReasonPhrase(),null);
			} else if (x==0) {
				return ResponseWrapper.success(HttpStatus.BAD_REQUEST.value(), "该项目信息不存在", null);
			} else {
				return ResponseWrapper.success(HttpStatus.BAD_REQUEST.value(), "该项目信息已被作废，不能进行更新",null);
			}
		} catch (Exception e) {
			e.printStackTrace();
			log.error(e.getMessage());
		}
		return ResponseWrapper.error(HttpStatus.INTERNAL_SERVER_ERROR.value(), HttpStatus.INTERNAL_SERVER_ERROR.getReasonPhrase(),"服务错误，请联系管理员");
	}

	/**
	 * @author WangZL
	 * @Description 项目信息-删除
	 * @param 项目信息ID
	 * @return 状态
	 */
	@DeleteMapping(value="/project/{xmId}")
	@ApiOperation(value = "项目信息-删除", notes = "参数：项目信息ID")
	public ResponseWrapper delete(@PathVariable("xmId") String xmId) {
		try{
			int x = projectService.delete(xmId);
			if(x>0) {
				return ResponseWrapper.success(HttpStatus.OK.value(), HttpStatus.OK.getReasonPhrase(),null);
			} else if (x==0) {
				return ResponseWrapper.success(HttpStatus.BAD_REQUEST.value(), "删除失败", null);
			} else {
				return ResponseWrapper.success(HttpStatus.BAD_REQUEST.value(), "删除失败,请项目信息ID是否正确", null);
			}
		} catch (Exception e) {
			e.printStackTrace();
			log.error(e.getMessage());
		}
		return ResponseWrapper.error(HttpStatus.INTERNAL_SERVER_ERROR.value(), HttpStatus.INTERNAL_SERVER_ERROR.getReasonPhrase(),"服务错误，请联系管理员");
	}

	/**
	 * @author WangZL
	 * @Description 项目信息-查询详情
	 * @param 项目信息ID
	 * @return 状态
	 */
	@GetMapping(value="/project/{xmId}")
	@ApiOperation(value = "项目信息-查询详情", notes = "参数：项目信息ID")
	public ResponseWrapper getProjectInfoById(@PathVariable("xmId") String xmId) {
		try{
			ProjectVO projectVO = projectService.getProjectInfoById(xmId);
			if(projectVO == null) {
				return ResponseWrapper.success(HttpStatus.BAD_REQUEST.value(), "获取失败，项目信息不存在", null);
			}
			return ResponseWrapper.success(HttpStatus.OK.value(), HttpStatus.OK.getReasonPhrase(), projectVO);
		} catch (Exception e) {
			e.printStackTrace();
			log.error(e.getMessage());
		}
		return ResponseWrapper.error(HttpStatus.INTERNAL_SERVER_ERROR.value(), HttpStatus.INTERNAL_SERVER_ERROR.getReasonPhrase(),"服务错误，请联系管理员");
	}

	/**
	 * @author WangZL
	 * @Description 项目信息-获取已发布最新项目信息
	 * @param
	 * @return 状态
	 */
	@PostMapping(value="/projectInfo")
	@ApiOperation(value = "项目信息-获取已发布最新项目信息", notes = "参数：")
	public ResponseWrapper getProjectInfo() {
		try{
			ProjectVO projectVO = projectService.getProjectInfo();
			if(projectVO == null) {
				return ResponseWrapper.success(HttpStatus.BAD_REQUEST.value(), "获取失败，项目信息不存在", null);
			}
			return ResponseWrapper.success(HttpStatus.OK.value(), HttpStatus.OK.getReasonPhrase(), projectVO);
		} catch (Exception e) {
			e.printStackTrace();
			log.error(e.getMessage());
		}
		return ResponseWrapper.error(HttpStatus.INTERNAL_SERVER_ERROR.value(), HttpStatus.INTERNAL_SERVER_ERROR.getReasonPhrase(),"服务错误，请联系管理员");
	}

	/**
	 * @author WangZL
	 * @Description 项目信息-发布
	 * @param 项目信息ID
	 * @return 状态
	 */
	@PostMapping(value="/updateReleaseStatus/{xmId}")
	@ApiOperation(value = "项目信息-发布", notes = "参数：项目信息ID")
	public ResponseWrapper updateReleaseStatus(@PathVariable("xmId") String xmId) {
		try{
			int x = projectService.updateReleaseStatus(xmId);
			if(x>0) {
				return ResponseWrapper.success(HttpStatus.OK.value(), HttpStatus.OK.getReasonPhrase(),null);
			} else if (x==0) {
				return ResponseWrapper.success(HttpStatus.BAD_REQUEST.value(), "删除失败", null);
			} else {
				return ResponseWrapper.success(HttpStatus.BAD_REQUEST.value(), "删除失败,请项目信息ID是否正确", null);
			}
		} catch (Exception e) {
			e.printStackTrace();
			log.error(e.getMessage());
		}
		return ResponseWrapper.error(HttpStatus.INTERNAL_SERVER_ERROR.value(), HttpStatus.INTERNAL_SERVER_ERROR.getReasonPhrase(),"服务错误，请联系管理员");
	}

}
